export declare const enum Token {
    Type = 255,
    PrecedenceStart = 8,
    Precedence = 3840,
    Keyword = 4096,
    Contextual = 12288,
    Reserved = 20480,
    FutureReserved = 36864,
    IsExpressionStart = 65536,
    IsIdentifier = 143360,
    IsInOrOf = 262144,
    IsLogical = 524288,
    IsAutoSemicolon = 1048576,
    IsPatternStart = 2097152,
    IsAssignOp = 4194304,
    IsBinaryOp = 8388608,
    IsUnaryOp = 16842752,
    IsUpdateOp = 33619968,
    IsMemberOrCallExpression = 67108864,
    IsStringOrNumber = 134217728,
    IsCoalesce = 268435456,
    IsEvalOrArguments = 537079808,
    IsClassField = 1073741824,
    IsEscaped = -2147483648,
    EOF = 1048576,
    Identifier = 208897,
    NumericLiteral = 134283266,
    StringLiteral = 134283267,
    RegularExpression = 65540,
    FalseKeyword = 86021,
    TrueKeyword = 86022,
    NullKeyword = 86023,
    TemplateContinuation = 67174408,
    TemplateSpan = 67174409,
    Arrow = 10,
    LeftParen = 67174411,
    LeftBrace = 2162700,
    Period = 67108877,
    Ellipsis = 14,
    RightBrace = 1074790415,
    RightParen = 16,
    Semicolon = 1074790417,
    Comma = 18,
    LeftBracket = 69271571,
    RightBracket = 20,
    Colon = 21,
    QuestionMark = 22,
    SingleQuote = 23,
    DoubleQuote = 24,
    Increment = 33619993,
    Decrement = 33619994,
    Assign = 1077936155,
    ShiftLeftAssign = 4194332,
    ShiftRightAssign = 4194333,
    LogicalShiftRightAssign = 4194334,
    ExponentiationAssign = 4194335,
    AddAssign = 4194336,
    SubtractAssign = 4194337,
    MultiplyAssign = 4194338,
    DivideAssign = 4259875,
    ModuloAssign = 4194340,
    BitwiseXorAssign = 4194341,
    BitwiseOrAssign = 4194342,
    BitwiseAndAssign = 4194343,
    LogicalOrAssign = 4194344,
    LogicalAndAssign = 4194345,
    CoalesceAssign = 4194346,
    TypeofKeyword = 16863275,
    DeleteKeyword = 16863276,
    VoidKeyword = 16863277,
    Negate = 16842798,
    Complement = 16842799,
    Add = 25233968,
    Subtract = 25233969,
    InKeyword = 8673330,
    InstanceofKeyword = 8411187,
    Multiply = 8391476,
    Modulo = 8391477,
    Divide = 8457014,
    Exponentiation = 8391735,
    LogicalAnd = 8913720,
    LogicalOr = 8913465,
    StrictEqual = 8390458,
    StrictNotEqual = 8390459,
    LooseEqual = 8390460,
    LooseNotEqual = 8390461,
    LessThanOrEqual = 8390718,
    GreaterThanOrEqual = 8390719,
    LessThan = 8456256,
    GreaterThan = 8390721,
    ShiftLeft = 8390978,
    ShiftRight = 8390979,
    LogicalShiftRight = 8390980,
    BitwiseAnd = 8390213,
    BitwiseOr = 8389702,
    BitwiseXor = 8389959,
    VarKeyword = 86088,
    LetKeyword = 241737,
    ConstKeyword = 86090,
    BreakKeyword = 20555,
    CaseKeyword = 20556,
    CatchKeyword = 20557,
    ClassKeyword = 86094,
    ContinueKeyword = 20559,
    DebuggerKeyword = 20560,
    DefaultKeyword = 20561,
    DoKeyword = 20562,
    ElseKeyword = 20563,
    ExportKeyword = 20564,
    ExtendsKeyword = 20565,
    FinallyKeyword = 20566,
    ForKeyword = 20567,
    FunctionKeyword = 86104,
    IfKeyword = 20569,
    ImportKeyword = 86106,
    NewKeyword = 86107,
    ReturnKeyword = 20572,
    SuperKeyword = 86109,
    SwitchKeyword = 86110,
    ThisKeyword = 86111,
    ThrowKeyword = 86112,
    TryKeyword = 20577,
    WhileKeyword = 20578,
    WithKeyword = 20579,
    ImplementsKeyword = 36964,
    InterfaceKeyword = 36965,
    PackageKeyword = 36966,
    PrivateKeyword = 36967,
    ProtectedKeyword = 36968,
    PublicKeyword = 36969,
    StaticKeyword = 36970,
    YieldKeyword = 241771,
    AsKeyword = 77932,
    AsyncKeyword = 209005,
    AwaitKeyword = 209006,
    ConstructorKeyword = 12399,
    GetKeyword = 209008,
    SetKeyword = 209009,
    AccessorKeyword = 12402,
    FromKeyword = 209011,
    OfKeyword = 471156,
    EnumKeyword = 86133,
    Eval = 537079926,
    Arguments = 537079927,
    EscapedReserved = -2147483528,
    EscapedFutureReserved = -2147483527,
    AnyIdentifier = 209018,
    PrivateIdentifier = 123,
    BigIntLiteral = 134283388,
    Coalesce = 276824445,
    QuestionMarkPeriod = 67108990,
    WhiteSpace = 127,
    Illegal = 128,
    CarriageReturn = 129,
    PrivateField = 130,
    Template = 131,
    Decorator = 132,
    Target = 209029,
    Meta = 209030,
    LineFeed = 135,
    EscapeStart = 136,
    JSXText = 137
}
export declare const KeywordDescTable: string[];
export declare const descKeywordTable: {
    [key: string]: Token;
};
