export type { ParsedDataURI } from './common';
/**
 * Returns a `Buffer` instance from the given data URI `uri`.
 *
 * @param {String} uri Data URI to turn into a Buffer instance
 */
export declare const dataUriToBuffer: (uri: string | URL) => import("./common").ParsedDataURI;
//# sourceMappingURL=index.d.ts.map