export declare const enum Chars {
    Null = 0,
    Backspace = 8,
    Tab = 9,
    LineFeed = 10,
    VerticalTab = 11,
    FormFeed = 12,
    CarriageReturn = 13,
    Space = 32,
    Exclamation = 33,
    DoubleQuote = 34,
    Hash = 35,
    Dollar = 36,
    Percent = 37,
    Ampersand = 38,
    SingleQuote = 39,
    LeftParen = 40,
    RightParen = 41,
    Asterisk = 42,
    Plus = 43,
    Comma = 44,
    Hyphen = 45,
    Period = 46,
    Slash = 47,
    Zero = 48,
    One = 49,
    Two = 50,
    Three = 51,
    Four = 52,
    Five = 53,
    Six = 54,
    Seven = 55,
    Eight = 56,
    Nine = 57,
    Colon = 58,
    Semicolon = 59,
    LessThan = 60,
    EqualSign = 61,
    GreaterThan = 62,
    QuestionMark = 63,
    UpperA = 65,
    UpperB = 66,
    UpperC = 67,
    UpperD = 68,
    UpperE = 69,
    UpperF = 70,
    UpperG = 71,
    UpperH = 72,
    UpperI = 73,
    UpperJ = 74,
    UpperK = 75,
    UpperL = 76,
    UpperM = 77,
    UpperN = 78,
    UpperO = 79,
    UpperP = 80,
    UpperQ = 81,
    UpperR = 82,
    UpperS = 83,
    UpperT = 84,
    UpperU = 85,
    UpperV = 86,
    UpperW = 87,
    UpperX = 88,
    UpperY = 89,
    UpperZ = 90,
    LeftBracket = 91,
    Backslash = 92,
    RightBracket = 93,
    Caret = 94,
    Underscore = 95,
    Backtick = 96,
    LowerA = 97,
    LowerB = 98,
    LowerC = 99,
    LowerD = 100,
    LowerE = 101,
    LowerF = 102,
    LowerG = 103,
    LowerH = 104,
    LowerI = 105,
    LowerJ = 106,
    LowerK = 107,
    LowerL = 108,
    LowerM = 109,
    LowerN = 110,
    LowerO = 111,
    LowerP = 112,
    LowerQ = 113,
    LowerR = 114,
    LowerS = 115,
    LowerT = 116,
    LowerU = 117,
    LowerV = 118,
    LowerW = 119,
    LowerX = 120,
    LowerY = 121,
    LowerZ = 122,
    LeftBrace = 123,
    VerticalBar = 124,
    RightBrace = 125,
    Tilde = 126,
    Delete = 127,
    NextLine = 133,
    NonBreakingSpace = 160,
    Ogham = 5760,
    EnQuad = 8192,
    EmQuad = 8193,
    EnSpace = 8194,
    EmSpace = 8195,
    ThreePerEmSpace = 8196,
    FourPerEmSpace = 8197,
    SixPerEmSpace = 8198,
    FigureSpace = 8199,
    PunctuationSpace = 8200,
    ThinSpace = 8201,
    HairSpace = 8202,
    ZeroWidthSpace = 8203,
    ZeroWidthJoiner = 8204,
    ZeroWidthNonJoiner = 8205,
    LineSeparator = 8232,
    ParagraphSeparator = 8233,
    NarrowNoBreakSpace = 8239,
    MathematicalSpace = 8287,
    IdeographicSpace = 12288,
    ZeroWidthNoBreakSpace = 65279,
    ByteOrderMark = 65519,
    NonBMPMax = 1114111,
    LeadSurrogateMin = 55296,
    LeadSurrogateMax = 56319,
    TrailSurrogateMin = 56320,
    TrailSurrogateMax = 57343,
    UTF16Max = 65535,
    NonBMPMin = 65536
}
