"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.severityLevels = void 0;
exports.severityLevels = {
    none: 0,
    low: 1,
    medium: 2,
    high: 3,
    critical: 4,
};
