export * as core from "../core/index.js";
export * from "./parse.js";
export * from "./schemas.js";
export * from "./checks.js";

export type { infer, output, input } from "../core/index.js";
export {
  globalRegistry,
  registry,
  config,
  $output,
  $input,
  $brand,
  function,
  clone,
  regexes,
  treeifyError,
  prettifyError,
  formatError,
  flattenError,
  toJSONSchema,
  TimePrecision,
  NEVER,
} from "../core/index.js";

export * as locales from "../locales/index.js";
/** A special constant with type `never` */
// export const NEVER = {} as never;

// iso
export * as iso from "./iso.js";
export {
  ZodMiniISODateTime,
  ZodMiniISODate,
  ZodMiniISOTime,
  ZodMiniISODuration,
} from "./iso.js";

// coerce
export * as coerce from "./coerce.js";
